<?php
namespace Models;

// Модель для работы с Фискальным регистратором
class Fr extends \DB\SQL\Mapper{
	public $db, $f3, $printer, $sets, $fr, $ticket_type=array('1'=>0, '2'=>1), $api_data;

	public function __construct($db, $f3, $printer='kkm', $open=true) {
		parent::__construct($db, 'fr_settings'); // подключаемся к таблице с настройками
		$this->f3=$f3;
		$this->db=$db;
		$this->receipt=array();
		$this->printer=$printer;
		$this->sets=$this->get_fr();
		if($this->sets['active']==1 && $this->sets['driver']!=''  && $open===true) {
			if(file_exists($this->sets['driver'].'/server.py')) {
				$this->fr=new Fr_python($f3, $this->sets);
			}
			elseif(file_exists($this->sets['driver'].'/server.php')) {
				$class=explode('/', $this->sets['driver']);
				$class=array_reverse($class);
				$model='\Models\\'.$class[0].'_Fr_php';
				if(!class_exists($model, false)) {
					require_once($this->sets['driver'].'/server.php');
				}
				$file='/server.config';
				if($printer=='pre' || $printer=='kitchen' || $printer=='bar') {
					if(file_exists($this->sets['driver'].'/'.$printer.'_server.config')) {
						$file='/'.$printer.'_server.config';
					}
				}
				$config=parse_ini_file($this->sets['driver'].$file);
				$this->fr=new $model($f3, $this->sets, $config);
			}
			else {
				echo 'Ошибка';
			}
		}
		$this->api_data=\Models\Controller::get_api_config();
	}

	// Список драйверов
	function get_drivers() {
		$drivers=array();
		$printers_dir=$this->f3->get('printers_dir');
		$dirs=scandir($printers_dir); // Ищем все драйвера
		if($dirs) {
			foreach ($dirs as $d) {
				$folder=$printers_dir.$d;
				$file=$folder.'/server.config';
				if(is_dir($folder) && file_exists($file)) {
					$config=parse_ini_file($file);
					if($config) {
						$drivers[$folder]=array(
							'title'=>$config['ServerName'],
							'path'=>$folder
						);
					}
				}
			}
		}

		return $drivers;
	}

	// Настройки ФР
	function get_fr() {
		$this->reset();
		$this->load(array("printer=?", $this->printer), array('limit'=>1));
		$sets=$this->cast();
		$sets['pays_type']=json_decode($sets['pays_type'], true);
		$sets['vat_type']=json_decode($sets['vat_type'], true);
		return $sets;
	}

	// Сохраняем настройки ФР
	function save_fr($post) {
		if($this->printer=='kkm' && in_array(0, $post['vat']) && $post['active']==1) {
			return array(
				'success'=>false,
				'txt'=>'vat_not_null'
			);
		}
		if($this->printer=='kkm') {
			$post['pays_type']=json_encode($post['pays']);
			$post['vat_type']=json_encode($post['vat']);
			unset($post['pays'], $post['vat']);
		}
		else {
			$post['pays_type']='';
			$post['vat_type']='';
		}
		if($post['fix_user']) {
			$post['fix_user']=trim($post['fix_user']);
		}
		$this->reset();
		$this->get_fr();
		$post['printer']=$this->printer;
		$this->copyFrom($post);
		if($this->save()===false) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful'
		);
	}

	// Открытие смены
	function open_session($print=1) {
		$user=$this->f3->get('SESSION.user.short_name');
		if($this->sets['fix_user']) {
			$user=$this->sets['fix_user'];
		}
		$strs="open_session;".$print.";".$user.";\n";
		if(!$this->sets['type'] && $print==1) {
			if($this->sets['header']) {
				$strs.=$this->sets['header']."\n";
			}
			$strs.="p;ОТКРЫТИЕ СМЕНЫ#kkm_br#".date("d-m-Y H:i").";\n";
		}
		//$strs.="p;#kkm_br#Касса: ".$this->api_data['rm_id']."#kkm_br#";
		//$strs.="Кассир ".$this->f3->get('SESSION.user.short_name').";\n";
		if($this->sets['footer'] && $print==1) {

			$strs.=$this->sets['footer']."\n";
		}
		
		$this->send_ajax($strs);
	}

	// Закрытие смены
	function z($change, $print=1) {
		$user=$this->f3->get('SESSION.user.short_name');
		if($this->sets['fix_user']) {
			$user=$this->sets['fix_user'];
		}
		$strs="z;".$print.";".$user.";\n";
		if(!$this->sets['type'] && $print==1) {
			$strs.=$this->report($change, 'z');
		}
		else {
			//$strs.="p;Касса: ".$this->api_data['rm_id']."#kkm_br#";
			//$strs.="Кассир ".$this->f3->get('SESSION.user.short_name').";\n";
			if($this->sets['footer'] && $print==1) {
				$strs.=$this->sets['footer']."\n";
			}
		}
		
		$this->send_ajax($strs);
	}

	// X-отчет смены
	function x($change) {
		$strs="x;\n";
		if(!$this->sets['type']) {
			$strs.=$this->report($change, 'x');
		}
		else {
			//$strs.="p;Касса: ".$this->api_data['rm_id']."#kkm_br#";
			//$strs.="Кассир ".$this->f3->get('SESSION.user.short_name').";\n";
			if($this->sets['footer']) {
				$strs.=$this->sets['footer']."\n";
			}
		}
		
		$this->send_ajax($strs);
	}

	function report($change, $type) {
		$types=array('x'=>'НУЛЕВОЙ ОТЧЕТ', 'z'=>'ОТЧЕТ С ГАШЕНИЕМ');
		$strs='';
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		$strs.="p;".$types[$type]."#kkm_br#";

		if($change) {
			$receipts=new \Models\Receipts($this->db, $this->f3);
			$report=$receipts->change_statistic($change['id']);
			$p=new \Models\Paymentmethods($this->db, $this->f3);
			$paymentmethods=$p->select_method();
			foreach ($paymentmethods as $p) {
				$pays[$p['id']]=array('title'=>$p['title'], 'cash'=>$p['cash']);
			}
			$strs.=date("d/m/Y H:i")."#kkm_br#";
			$strs.="Смена ".$change['id']."#kkm_br#";
			$strs.="Начало смены ".date("d/m/Y H:i", $change['time_start'])."#kkm_br#";
		}

		if($type=='z') {
			$strs.="Конец смены ".date("d/m/Y H:i");
		}
		$strs.="#kkm_br#";

		if($change) {
			$types=array(
				$this->f3->get('receipt_sales')=>'Продажи',
				$this->f3->get('receipt_return')=>'Возвраты',
				$this->f3->get('receipt_cashin')=>'Внесение',
				$this->f3->get('receipt_cashout')=>'Изъятие'
			);
			foreach ($report as $type => $r) {
				if(!$types[$type]) continue;
				$strs.="#kkm_br#".$types[$type]." = ".$r['count']."#kkm_br#";
				foreach ($pays as $i => $p) {
					$strs.="  ".$p['title']." = ".number_format(abs($r['pay'][$i]), 2, '.', '')."#kkm_br#";
					if($p['cash']==1) {
						if($type==$this->f3->get('receipt_return'))$total-=$r['pay'][$i];
						else $total+=$r['pay'][$i];
					}
				}
				$strs.="  Сумма = ".number_format(abs($r['sum']), 2, '.', '')."#kkm_br##kkm_br#";
			}
			$strs.="#kkm_br#Наличность в кассе = ".number_format($total, 2, '.', '');
			$strs.=";\n";
		}
		//$strs.="p;Касса: ".$this->api_data['rm_id']."#kkm_br#";
		//$strs.="Кассир ".$this->f3->get('SESSION.user.short_name').";\n";
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		return $strs;
	}

	// Добавляем чек
	function ticket($t, $lines, $discount=0, $sales_receipts=0, $print=1, $egais=null) {
		$this->receipt=$t['receipt'];
		$user=$this->f3->get('SESSION.user.short_name');
		if($this->sets['fix_user']) {
			$user=$this->sets['fix_user'];
		}
		$vat_type=$this->f3->get('vat_type');
		$psr=1; // Признак способа расчета по умолчанию
		$ppr=1; // Признак предмета расчета по умолчанию
		$section=1; // Секция

		$strs='';
		// Заголовок
		if($this->sets['header'] && $print==1) {
			$strs.=$this->sets['header']."\n";
		}

		if($sales_receipts && $print==1) {
			$strs.="p;Товарный чек;\n";
		}

		$strs.='b;'.($this->ticket_type[$t['type']]).';Кассир '.$user.";".$print.";\n"; // Открываем чек

		if($t['client']) {
			$strs.="set_tlv;1008;5;".$t['client'].";\n";
		}

		// Товары
		$discount_sum=0;
		foreach ($lines as $l) {
			if($l['loyalty']==0 && !$l['discount']) {
				$l['discount']=$discount;
			}
			$title=mb_substr(str_replace(';', ',', $l['product_title']), 0, 100);
			$price=$l['price']+($l['price']*($l['vat']/100))*max(0, $l['vat_type']);
			$d_sum=(float) $price*((float) $l['discount']/100);
			$price-=$d_sum;
			if($l['line_discount']>0) {
				$price0=$l['price0']+($l['price0']*($l['vat']/100))*max(0, $l['vat_type']);
				$d_sum=(float) $price0*((float) $l['line_discount']/100);
			}
			$discount_sum+=$d_sum*$l['count'];
			$price*=100;
			$price=round($price);
			$vt=$vat_type;
			if(!isset($l['ppr'])) $l['ppr']=$ppr;
			if($l['vat_type']==0) $vt=$this->sets['vat_type'][0];
			elseif($l['vat_type']!=0 && $l['vat']==0) $vt=$this->sets['vat_type'][1];
			elseif($l['vat_type']==1 && $l['vat']>0 && $l['vat']<=10) $vt=$this->sets['vat_type'][2];
			elseif($l['vat_type']==-1 && $l['vat']>0 && $l['vat']<=10) $vt=$this->sets['vat_type'][3];
			elseif($l['vat_type']==1 && $l['vat']>10) $vt=$this->sets['vat_type'][4];
			elseif($l['vat_type']==-1 && $l['vat']>10) $vt=$this->sets['vat_type'][5];
			if($l['mark']!='') {
				$mark=json_decode($l['mark'], true);
				$i=1;
				if(!$l['pg'])$l['pg']='other';
				foreach ($mark as $m) {
					$count=1;
					if($i==count($mark))$count=$l['count']-$i+1;
					$i++;
					$strs.='smde;'.$title.'{{marking}}'.$m.'{{marking}}'.$l['pg'].';'.$price.';'.round($count*1000).';'.$vt.';'.$section.';';
					if($this->sets['version']=="1.1") {
						$strs.=$psr.';'.$l['ppr'].';';
					}
					$strs.="\n";
					//$strs.="set_tlv;1162;5;".$m.";\n";
				}
			}
			else {
				$strs.='smde;'.$title.';'.$price.';'.round($l['count']*1000).';'.$vt.';'.$section.';';
				if($this->sets['version']=="1.1") {
					$strs.=$psr.';'.$l['ppr'].';';
				}
				$strs.="\n";
			}
		}

		// оплата
		$strs.='tmde;';
		$pays=json_decode($t['payment'], true);
		// 1 способ оплаты, только для ФФД 1.05
		if($this->sets['version']=="1.05" && count($pays)==1) {
			$strs.=round($pays[0]['sum']*100).';'.max(0, $this->sets['pays_type'][$pays[0]['method']]).';';
		}
		// несколько способов оплаты или ФФД 1.1
		else {

			foreach ($pays as $p) {
				$mpay[$this->sets['pays_type'][$p['method']]]=$p['sum'];
			}

			if($this->sets['version']=='1.05') {
				$pay_count=4;
			}
			else {
				$pay_count=5;
			}

			for($i=0; $i<$pay_count; $i++) {
				if(!$mpay[$i])$mpay[$i]=0;
				$strs.=max(round($mpay[$i]*100), 0).';';
			}

			// Для ФФД 1.05 указываем налог для всех видов оплат
			if($this->sets['version']=="1.05") {
				for($i=0; $i<$pay_count; $i++) {
					//$strs.=$this->sets['sno'].';';
					$strs.="0;";
				}
			}
			// Для ФФД 1.1 указываем налог только 1 раз
			else {
				$strs.=$this->sets['sno'].';';
			}
		}
		$strs.="\n";

		if($egais && isset($egais['url']) && isset($egais['sign'])) {
			$strs.="p;".$egais['sign'].";\n";
			$strs.="qr;".$egais['url'].";\n";
			$strs.="p;".$egais['url'].";\n";
		}

		if($print==1) {
			/*$strs.="p;Позиций#kkm_right#".count($lines);
			if($discount_sum>0) {
				$strs.="#kkm_br#Скидка#kkm_right#".number_format($discount_sum, 2, '.', '');
			}
			$strs.="#kkm_br#";
			$strs.="Касса: ".$this->api_data['rm_id'];
			$strs.="#kkm_br#".date("d.m.Y H:i")." N ".$t['id']."\n";
			$strs.="Кассир ".$this->f3->get('SESSION.user.short_name').";\n";*/

			if($this->sets['footer']) {
				$strs.=$this->sets['footer']."\n";
			}
		}
		
		$this->send_ajax($strs);
	}

	// Внесение в кассу
	function cashin($change, $sum) {
		$user=$this->f3->get('SESSION.user.short_name');
		if($this->sets['fix_user']) {
			$user=$this->sets['fix_user'];
		}
		$strs="imde;".($sum*100).";".$user.";\n";
		if(!$this->sets['type']) { 
			// Заголовок
			if($this->sets['header']) {
				$strs.=$this->sets['header']."\n";
			}
			if($sum>=0) {
				$strs.="p;ВНЕСЕНИЕ В КАССУ";
			}
			else {
				$strs.="p;ИЗЪЯТИЕ ИЗ КАССЫ";
			}
			$strs.="#kkm_br#";
			$strs.="Сумма = ".number_format(abs($sum), 2, '.', '')."#kkm_br#";
			//$strs.="Касса: ".$this->api_data['rm_id']."#kkm_br#";
			//$strs.="Кассир ".$this->f3->get('SESSION.user.short_name')."#kkm_br#";
			$strs.="Смена ".$change.";\n";
			//$strs.=date("d-m-Y H:i").";\n";
		}
		/*else {
			$strs.="p;Касса: ".$this->api_data['rm_id']."#kkm_br#";
			$strs.="Кассир ".$this->f3->get('SESSION.user.short_name').";\n";
		}*/
		// подвал
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		
		$this->send_ajax($strs);
	}

	// Обнуление не пробитого чека
	function reset_ticket() {
		$strs="g;";
		$this->send_ajax($strs);
		return array(
			'success'=>true,
			'txt'=>'ticket_reset'
		);
	}

	// Запрос полного статуса ККМ
	function status_kkm() {
		$strs="d;";
		$this->fr->send_ajax($strs, null, true);
	}

	// Продолжить печать, после аварийной остановки
	function continue_print() {
		$strs="continue_print;";
		$this->f3->set('hold', false);
		$this->send_ajax($strs);
		return array(
			'success'=>true,
			'txt'=>'print_continue'
		);
	}

	// Отчет по секциям
	function rep_sections() {
		$strs="rep_sections;";
		$this->send_ajax($strs);
	}

	// Печать копии последнего чека
	function c() {
		$strs='';
		// Заголовок
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		$strs.="c;\n";
		// Подвал
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		//$strs.="p;Касса: ".$this->api_data['rm_id']."#kkm_br#";
		//$strs.="Кассир ".$this->f3->get('SESSION.user.short_name').";\n";
		
		$this->send_ajax($strs);
	}

	// Печать пречека или товарного чека
	function print($receipt) {
		$this->receipt=$receipt;
		$strs='';
		// Заголовок
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}

		if($this->printer!='kitchen' && $this->printer!='bar') {
			$strs.='p;';
			// Товарный чек или пречек
			if($receipt['time']>0) {
				$strs.='#kkm_center#ТОВАРНЫЙ ЧЕК#kkm_br#';
			}
			else {
				$strs.='#kkm_center#ПРЕДВАРИТЕЛЬНЫЙ ЧЕК#kkm_br#';
			}
			if((int)$this->sets['one_line']==0) {
				$strs.=";\n";
			}
		}
		else {
			if((int)$this->sets['one_line']==1) {
				$strs.='print_font;2;';
			}
		}

		$sum=0;
		$discount_sum=0;
		// Позиции в чеке
		foreach ($receipt['receiptlines'] as $l) {
			// Цена
			if($l['loyalty']==0 && !$l['discount']) {
				$l['discount']=$receipt['discount'];
			}
			$price=$l['price']+($l['price']*($l['vat']/100))*max(0, $l['vat_type']);
			$price=round($price, 2);

			$d_sum=$price*($l['discount']/100);
			$price-=$d_sum;
			if($l['line_discount']>0) {
				$price0=$l['price0']+($l['price0']*($l['vat']/100))*max(0, $l['vat_type']);
				$d_sum=(float) $price0*((float) $l['line_discount']/100);
			}
			$discount_sum+=$d_sum*$l['count'];
			$sum+=$price*$l['count']; // Сумма чека
			$l['product_title']=str_replace(';', ',', $l['product_title']);
			if($l['parent_id']>0) {
				$l['product_title']=' - '.$l['product_title'];
			}
			if($this->printer=='kitchen' || $this->printer=='bar') {
				if((int)$this->sets['one_line']==1) {
					$strs.='#kkm_br#';
				}
				else {
					if($l['parent_id']>0) {
						$strs.='p;';
					}
					else {
						$strs.='print_font;2;';
					}
				}
				$strs.=$l['product_title'].'#kkm_br##kkm_right#X'.number_format($l['count'], 3, '.', '');
			}
			else {
				if((int)$this->sets['one_line']==1) {
					$strs.='#kkm_br#';
				}
				else {
					$strs.='p;';
				}
				$strs.=$l['product_title'].'#kkm_br##kkm_right#'.number_format($l['count'], 3, '.', '').'X'.number_format($price, 2, '.', '').'='.number_format($price*$l['count'], 2, '.', '');
			}

			// НДС товара
			if($l['vat_type']!=0) {
				$vp=(($l['price']/(100+$l['vat']*abs(min(0, $l['vat_type']))))*$l['vat'])*$l['count'];
				$vp=round($vp, 2);
				$vat[$l['vat']]+=$vp; // Сумма НДС по чеку
				if($this->printer!='kitchen' && $this->printer!='bar') {
					$strs.='#kkm_br#НДС#kkm_right#'.$l['vat'].'%='.number_format($vp, 2, '.', '').'#kkm_br#';
				}
			}
			if((int)$this->sets['one_line']==1) {
				$strs.='#kkm_br#';
			}
			else {
				$strs.=";\n";
			}
		}

		if($this->printer!='kitchen' && $this->printer!='bar') {
			$strs.='p;#kkm_bold#ИТОГ#kkm_right#='.number_format($sum, 2, '.', '').'#kkm_br#';
		}

		// Если чек пробит
		if($this->printer!='kitchen' && $this->printer!='bar' && $receipt['time']>0 && $receipt['pays']) {
			$cash=0;
			// Оплата
			$strs.='ОПЛАТА#kkm_br#';
			foreach ($receipt['pays'] as $p) {
				$cash+=$p['sum'];
				$strs.=$p['title'].'#kkm_right#='.number_format(round($p['sum'], 2), 2, '.', '').'#kkm_br#';
			}
			// Сдача
			$strs.='Сдача#kkm_right#='.number_format($cash-$sum, 2, '.', '').'#kkm_br#';
			$sno=array('01'=>'ОСН','02'=>'УСН','04'=>'УСН','08'=>'ЕНВД','10'=>'ЕСХН ','20'=>'Патент');
			// СНО
			$strs.='СНО#kkm_right#'.$sno[$this->sets['sno']].'#kkm_br#';
			// НДС по чеку
			if($vat) {
				$strs.='НДС#kkm_br#';
				foreach ($vat as $p=>$v) {
					$strs.='#kkm_right#'.number_format($p, 2, '.', '').'% = '.number_format($v, 2, '.', '').'#kkm_br#';
				}
			}
			// Кассир и смена
			$strs.='Кассир#kkm_br#';
			$user=$receipt['user']['short_name'];
			if($this->sets['fix_user']) {
				$user=$this->sets['fix_user'];
			}
			$strs.=$user.'#kkm_br#';
			$strs.='Смена '.$receipt['changes'].'#kkm_br#';
		}

		if($this->printer!='kitchen' && $this->printer!='bar') {
			$strs.="Позиций#kkm_right#".count($receipt['receiptlines'])."#kkm_br#";

			if($discount_sum>0) {
				$strs.="Скидка#kkm_right#".number_format($discount_sum, 2, '.', '').'#kkm_br#';
			}
			$strs.='ЧЕК '.$receipt['id'].'#kkm_br#';
		}

		/*if($receipt['time']>0) {
			$strs.=date('d-m-Y H:i', $receipt['time']);
		}*/
		$strs.=";\n";

		//$strs.="Касса: ".$this->api_data['rm_id']."#kkm_br#";
		//$strs.="Кассир ".$this->f3->get('SESSION.user.short_name').";\n";

		// Подвал
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		
		$this->send_ajax($strs);
	}

	// Замена переменных
	function replace_strs($strs) {
		$strs=str_replace('#kkm_rm#', $this->api_data['rm_id'], $strs);
		$user=$this->f3->get('SESSION.user.short_name');
		$print_user=$user;
		if($this->sets['fix_user']) {
			$user=$this->sets['fix_user'];
		}
		$strs=str_replace('#kkm_user#', $user, $strs);
		$strs=str_replace('#print_user#', $print_user, $strs);
		$strs=str_replace('#date#', date('d.m.Y'), $strs);
		$strs=str_replace('#time#', date('H:i:s'), $strs);
		if($this->receipt) {
			$strs=str_replace('#receipt_number#', $this->receipt['id'], $strs);
			$strs=str_replace('#table_number#', $this->receipt['table_id'], $strs);
			$strs=str_replace('#room_number#', $this->receipt['table']['room'], $strs);
			$strs=str_replace('#table#', $this->receipt['table']['title'], $strs);
			$strs=str_replace('#room#', $this->receipt['table']['room_title'], $strs);
		}
		else {
			$strs=str_replace('#receipt_number#', '', $strs);
			$strs=str_replace('#table_number#', '', $strs);
			$strs=str_replace('#room_number#', '', $strs);
			$strs=str_replace('#table#', '', $strs);
			$strs=str_replace('#room#', '', $strs);
		}
		if($change_id) {
			$strs=str_replace('#change_number#', $change_id, $strs);
		}
		if($receipt_id) {
			$strs=str_replace('#receipt_number#', $receipt_id, $strs);
		}
		return $strs;
	}

	// Отправка запроса
	function send_ajax($strs) {
		$strs=$this->replace_strs($strs);
		if($this->f3->get('GET.continue_print')==1) {
			$this->f3->clear('GET.continue_print');
			$this->continue_print();
			return;
		}
		if(!$this->sets['active'] || $this->f3->get('hold')) return true;
		$this->fr->send_ajax($strs);
	}

	// Состояние смены
	function change_status() {
		// closed - закрыта
        // opened - открыта
        // expired - истекла
		if(!$this->sets['active'] || !$this->sets['type']) return 'opened'; // Если ККМ не используется или используется чекопечатающий принтер
		$status=$this->fr->change_status();
		return $status;
	}
}
